#!/usr/bin/env /proj/axaf/bin/perl

# --8<--8<--8<--8<--
#
# Copyright (C) 2006 Smithsonian Astrophysical Observatory
#
# This file is part of mstbuild
#
# mstbuild is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# mstbuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#
# -->8-->8-->8-->8--


use strict;
use warnings;

use FindBin;
use File::Basename;

use autouse 'File::Spec::Functions' => qw( catfile catdir );

use Module::Runtime qw[ use_module ];
use Capture::Tiny qw[ capture ];

my $pkgvdatadir;

BEGIN {

    # see if we can get to the configuration variables
    eval "use mstbuild::ConfigVars";

    # nope.  must be off the beaten track
    if ( $@ )
    {
        foreach  ( $ENV{mstbuild_pkgvdatadir},
                   '/proj/axaf/simul/share/mstbuild-3.2.23',
                   "$FindBin::RealBin/../share/mstbuild-3.2.23",
                   "$FindBin::RealBin/../../share/mstbuild-3.2.23"
                 ) {

            next unless defined $_ && -d $_;
            $pkgvdatadir = $_;
        }

        die( "Unable to find module directory for mstbuild 3.2.23\n" )
          unless defined $pkgvdatadir;

	eval "use lib '$pkgvdatadir/lib'";
	die $@ if $@;
    }
}


use mstbuild::ConfigVars qw( :all );

use mstbuild::setup;
use mstbuild::util;

use mstbuild::Config;



eval { main() };

do { warn $@; exit 1 } if $@;

exit 0;


##############################################################################

sub main
{

    # read config file
    my $cfg = mstbuild::Config->new( 'mstbuild.cfg' );

    my %opt = parse_args();

    help(1) if $opt{help};
    help(2) if $opt{man} || $opt{usage};

    if ( $opt{version} )
    {
	print VERSION(), "\n";
	return;
    }
    # this is hideous, but it must be removed so that
    # it doesn't interfere with a command specific --version option.
    delete $opt{version};

    # if no argument given, they obviously need help
    if ( ! @ARGV )
    {
	docmd( \%opt, 'help' );
    }

    else
    {
	docmds( \%opt );
    }
}

#############################################################################


sub docmds
{
    my ( $opt ) = @_;
    # iterate over commands in @ARGV;

    while ( @ARGV )
    {
	my $cmd = shift( @ARGV );
	docmd( $opt, $cmd );
    }
}


my %CmdAlias;

INIT {
    %CmdAlias =
      (
        configure         => 'config',
       'maintainer-clean' => 'maintclean',
        bootstrap         => 'boot',
      );
};


sub docmd
{
    my ( $opt, $cmd ) = @_;

    # make keyboarding easier
    $cmd = $CmdAlias{$cmd} if exists $CmdAlias{$cmd};

    # create a valid subroutine name
    $cmd =~ s/\W/_/g;

    my $CLASS = eval { use_module( "mstbuild::cmd::$cmd" ) };

    if ( $@ )
    {
	die( $opt->{verbose}
	     ? "unknown command: $cmd ($@)\n"
	     : qq[unknown command: $cmd -- run "$0 -v $cmd"  for more information\n]
	   );
    }

    $CLASS->load_subpkgs;
    $cmd = $CLASS->new;

    # run setup if required, but not more than once
    mstbuild::setup::setup($opt, $cmd->get_perform_setup );
    $cmd->parse_options( $opt );
    $cmd->process;
}

#############################################################################

sub parse_args
{

    my $config = $ENV{CONFIG_SITE} || CONFIG_SITE;

    my $option_specs =
    {
     arch                => [ '!',      ARCH ],
     config              => [ '=s',     $config ],
     define	         => [ '|D=s%', {} ],
     destdir             => [ '=s',       ],
     dryrun              => [ '|n+',    0 ],
     exec_prefix         => [ '=s',       ],
     graft		 => [ '=s',     GRAFT ],
     help                => [             ],
     localdir		 => [             ],
     makeflags           => [ '|m=s@', [] ],
     pdefine		 => [ '|P=s%', {} ],
     prefix              => [ '=s'        ],
     pkgdatadir		 => [ '=s',    PKGDATADIR ],
     root                => [ '=s',    ROOT ],
     target              => [ '|t!',      ],
     target_dir          => [ '=s',       ],
     target_link         => [ '!',        ],
     Target              => [             ],
     uenvs               => [ '!',      1 ],
     man                 => [             ],
     mstenv		 => [ '!',	1 ],
     usage               => [             ],
     verbose             => [ '|v'        ],
     version             => [             ],
    };


    delete $option_specs->{localdir} if MSTLOCALDIR eq '';

    if ( defined $ENV{MSTBUILD_ARGS} )
    {
	require Text::ParseWords;
	unshift @ARGV, Text::ParseWords::shellwords( $ENV{MSTBUILD_ARGS} );
    }

    my $cfg = mstbuild::Config->new;

    my %optval = $cfg->parse_options( {}, [ [global => $option_specs] ] );

    # extricate global options
    my %opt = %{$optval{global}};

    $opt{buildtarget} = $cfg->opt->{buildtarget};

    # if -T is set, then -t must be set.
    $opt{target} = $opt{Target} if $opt{Target};

    # use buildtarget directories if arch specific, unless
    # otherwise specified.
    $opt{target} = $opt{arch} unless defined $opt{target};

    $opt{target_link} = 1
      if $opt{target} && $opt{target_dir} && ! defined $opt{target_link};

    $opt{target_dir} = $opt{buildtarget}
      unless defined $opt{target_dir};

    if ( $opt{localdir} )
    {
	my ( $stdout, $stderr, $exit ) = capture {
	    # want a local dir, but will use /tmp if one is not in the
	    # database
	    system( MSTLOCALDIR, qw[ -F /tmp ] );
	};

	die( "error getting local directory: $stderr\n" )
	  if $exit;

	chomp( $opt{localdir} = $stdout );
	$opt{target_link} = 1;
    }

    $opt{graft} = '' if $opt{graft} =~ /^(off|no)$/i;

    if ( $opt{graft} )
    {
	mstbuild::util::set_dryrun($opt{graft},
			       mstbuild::util::get_dryrun( 'graft' ) );

    }

    return %opt;
}

##############################################################################

sub help
{
  my ( $verbose ) = @_;

  # verbosity = 2 causes pod2usage to call perldoc, which already pages
  require IO::Pager::Page if $verbose < 2;
  require Pod::Usage;
  Pod::Usage::pod2usage ( { -exitval => 0, -verbose => $verbose } );
}



__END__

=pod

=head1 NAME

mstbuild - manage the build environment and process for MST software

=head1 SYNOPSIS

mstbuild [<config options>] [<global options>] command [<command options>] [command [<command options>]]

=head1 DESCRIPTION

B<mstbuild>  is primarily  a front end  to a  variety of GNU  autotool
packages,  with  additional functionality   to  implement  local build
conventions.  It runs a subset of commands for initializing,
bootstrapping, configuring, and making an autotooled package.

It provides sensible, host-specific defaults for compilation flags as
well as installation directories.  It provides these to the autotools
through both command line arguments and environmental variables.

=head1 OPTIONS AND ARGUMENTS

B<mstbuild> takes a list of global options followed by one or more
commands, which may have options of their own.

Options are specified using a getopt style interface.  Long names are
available when preceded with double hyphens C<-->, in which case only
the minimal number of characters are required.  Options which take
values may be separated from those values with the C<=> character.

Environmental variables may be used to specify additional arguments
to the command line:

=over

=item *

C<MSTBUILD_ARGS0> specifies B<configuration options>.  Its
contents are processed before any other command line arguments

=item *

C<MSTBUILD_ARGS> specifies other types of options.  Its contents
are processed I<after> any configuration options and I<before> any
other arguments  on the command line.

=back

In the following descriptions, C<$ROOT> refers to the value of the
B<--root> argument.

=head2 Configuration Options

These options must occur at the very beginning of the command line.

=over

=item B<--cwd> F<dir>

Change to the specified directory before running B<mstbuild>.

=item B<--configfile>|B<--cfg> F<file>

The name of the configuration file to load.  This defaults to
F<mstbuild.cfg>.

=item B<--buildtarget> I<target>

The hardware/software platform to compile for. The default is
the B<sysarch> platform designated as C<platform_os_generic>.
The target name may be a symbolic name recognized by B<sysarch>
(e.g. C<platform_generic>, etc ).

=back

=head2 Global Options

=over

=item B<--config> I<path>

The path to a local B<autoconf> site configuration file.  This
defaults to either the environmental variable C<CONFIG_SITE>, or
C</proj/axaf/share/config.site>, in that order.  The value may contain strings of the
form C<$variable>, which will be replaced by the value of the
variable.  Available variables are C<buildtarget> (which will be
replaced by the value of the B<--buildtarget> option) as well as any
current environmental variable.


=item B<--define>|B<-D> I<VAR>=I<value>

Define the specified environmental variable, setting it equal to the
specified value. The value of the B<--nouenv> option has no affect on
this option.  The value may contain variable references of the form
C<${var}>; these will be interpolated by B<mstbuild>.  The available
variables are those generated by the following command:

  mst_envs --nofullpath -ldlibrarypath

The value of the following options are also interpolated

  root
  buildtarget
  prefix
  exec_prefix
  destdir
  perl_archname  (Perl Config variable)


This option may be specified more than once.  See also the
B<--pdefine> option.

=item B<--localdir>

This option indicates that all steps after C<bootstrap> are run in a
separate I<local> directory.  The directory used is

   mstlocaldir/package/buildtarget

where I<mstlocaldir> is the output from the B<mstlocaldir> program,
I<package> is the name of the package, and I<buildtarget> is the value
of the B<--buildtarget> option.  This option forces B<--target_link>
to be true.  It has no effect on the state of the B<--target> option.
See B<--target_dir> for more control over this behavior.

If B<mstlocaldir> is not available, this option is disabled;
B<mstbuild> won't even know it exists.

=item B<--noarch>

This indicates that the package is not architecture specific, and sets

  exec_prefix = prefix

It overrides any command line attempts to specify C<exec_prefix>.

=item B<--nomstenv>

Normally B<mstbuild> will use the B<MST_Envs> package (if it is
installed) to find dependencies in the "standard" places in the MST
environment.  This option instructs B<mstbuild> to not use the
package.

=item B<--nouenv>

Indicate that certain environmental variables will be ignored.  See
L<ENVIRONMENTAL VARIABLES> for the list.

=item B<--pdefine>|B<-P> I<VAR>=[B<position>]I<value>

Append or prepend to the specified environmental variable I<VAR>,
which is assumed to be a colon delimited list of paths.  By default
(if B<position> is not specified) the value is put at the end of the
list.  B<position> may either by the character C<+> (append) or C<-> (prepend).

The value of the B<--nouenv> option has no affect on this option.  The
value may contain variable references of the form C<${var}>; these
will be interpolated by B<mstbuild>.  The available variables are
those generated by the following command:

  mst_envs --nofullpath -ldlibrarypath

The value of the following options are also interpolated

  root
  buildtarget
  prefix
  exec_prefix
  destdir
  perl_archname  (Perl Config variable)

This option may be specified more than once.  See also the B<--define>
option.  This option is processed after th B<--define> option.

=item B<--target>|B<-t>

If specified, all steps after C<bootstrap> are run in a separate
directory given by B<--target_dir> (see also B<--localdir>).  This
defaults to on. Specify B<--notarget> to keep the build in the current
directory.

=item B<--target_dir> F<dir>

If B<--target> is specified, this option provides the name of the 
subdirectory.  It defaults to the value of the B<--buildtarget>
option.

=item B<--target_link>

If B<--target> is specified, create a symbolic link (named after the
value of the B<--buildtarget> option) from the target directory to the
current directory.  This defaults to true if B<--target> and
B<--target_dir> are set by the user.

=item B<--Target>|B<-T>

This is the similar to the B<--target> option, except that the subdirectory
will be emptied before being used.

=item B<--dryrun>|B<-n>

If specified once, B<mstbuild> will echo the commands it would
normally perform, but will not actually execute them.  If specified
twice, any command which recognizes a "dry-run" option will be
executed with that option.

=item B<--makeflags>|B<-m> I<make flags>

This option allows one to specify additional flags to be passed to the
B<make> command. Use this with extreme caution!  B<-m> may be repeated.

=item B<--man>|B<--usage>

print full instructions and exit

=item B<--help>

print an abbreviated help message and exit

=item B<--version>

print version and exit

=back

=head3 Installation and Grafting options

Values for the following options may include strings of the form
C<$variable>

  root prefix exec_prefix

These strings will be replaced by the value of the corresponding variable,
which may be an environmental variable or the value of the similarly
named option.  Values from the following options are available:

  buildtarget root prefix exec_prefix

In order to reduce confusion, the options are processed in the above
order, so that one can write the following:

  --prefix=${root}

and get a sensical answer.

=over

=item B<--graft> F<path>

The path to the B<graft> executable.  This defaults to what was
discovered when B<mstbuild> was built.  If B<graft> is not available,
the package will not be configured with grafting in mind.  Set this to
C<no> or C<off> to force grafting off if it defaults to on.

=item B<--root> I<path>

The root directory off of which all destination paths originate.  This
defaults to the C<${AXAF_SIMUL_ROOT}>.  This also defines where B<pkg-config> will look
for dependencies.

=item B<--destdir> F<path>

Sets the C<DESTDIR> variable for B<make install> if grafting is on.  It
defaults to C<$ROOT/pkgs>.  Do not set this unless you are very brave.

=item B<--prefix> F<prefix>

With grafting on, sets the C<prefix> variable for B<make install>.
With grafting off, sets the C<--prefix> option for B<configure>. Do
not set this unless you are very brave.

=item B<--exec_prefix> F<path>

With grafting on, sets the C<exec_prefix> variable for B<make
install>. With grafting off, sets the C<--exec-prefix> option for
B<configure>.  Do not set this unless you are very brave.

=back


=head2 Template Options

Some commands take options which modify variables available to the
template files.  These options are:

=over

=item B<--bugreport> I<string>

This specifies the value for the template C<bugreport> variable.
It defaults to that in F<templates.cfg>.

=item B<--copyrightholder> I<string>

This specifies the value for the template C<copyrightholder> variable.
It defaults to that in F<templates.cfg>.

=item B<--emailcontact> I<string>

This specifies the value for the template C<emailcontact> variable.
It defaults to that in F<templates.cfg>.

=item B<--license_type> I<license>

Specify the license to copy.  The available licenses are:

  GPL2
  GPL3
  SAO

The default is C<GPL3>.

=item B<--suffix> I<string>

A suffix to apply to the filenames of all files created by the
B<chkstds> or B<init> options. Directory names are unaffected.  A
preceding C<.> is I<not> automatically added.

=item B<--template_dir> I<dir>

Where to copy the templates from.  To use this requires more
information than is given here.

=item B<--version> I<string>

This specifies the value for the template C<version> variable.  The
default vaule is read from F<configure.ac> (if that exists) or is
C<1.0.0>.

=back


=head2 Commands

Commands may have options; these are of the same form as the global
options.  A command's options directly follow it on the command line.
Some commands are conglomerations of multiple commands and accept
options for all of the commands they perform.  For example,
B<bootstrap> also runs B<snippets>, so accepts the B<--copy> option.
If some of these commands have similarly named options, this can cause
confusion.  For example, if a conglomerate command B<c> runs commands
B<a> and B<b> and both have an option named B<--args>, then in the
following command line,

  c --args foo

it's uncertain whether B<a> or B<b> gets the C<--args foo> option.  To
solve this problem, conglomerate command options with the same names
must be prefixed with their command names.  For example,

  c --a-args foo --b-args goo


Typically commands are run in the following sequence:

  bootstrap
  configure
  make
  check
  dist
  distcheck
  install
  graft
  export

To see the actual shell command lines which will be executed, use the
B<--dryrun> option.

=over

=item B<help>

=for brief show help for commands

  help [options] [ command_name | options [ option_type] ]

show help.

C<help> takes the following options:

=over

=item B<--page>

Use a pager to show the help information.

=back

It take zero or more arguments:

=over

=item B<help>

shows the list of commands.

=item B<help> C<command>

shows help for C<command>.

=item B<help> options I<[args]>

With no additional arguments, this shows the global options.  The
following arguments are available:

=over

=item config

shows the configuration options

=item template

shows the template options

=back

=back

=item B<init>

=for brief create a skeleton package

  init [I<options>] [I<package name>]

B<init> creates a skeleton package by copying template files to a
directory of the given name (which should be the package name).  If
the directory does not exist, it is created.  If no package name is
specified, the skeleton is created in the current directory.  If a
F<configure.ac> file is in the current directory, that is scanned for
the package name and version, else the package name is set to that of
the current directory.

Options:

Note that C<init> also takes template options. See L<Template Options>
or run

  mstbuild help options template


=over

=item B<--force>

Force files to be created, even if they already exist.  Backups
will be created for existant files.

=item B<--package_type|-t> I<package type>

This indicates what type of package is to be initialized.  The
available choices are:

=over

=item C<perl_prog> - a perl program

=item C<prog> - a compiled (C/C++) program

=item C<lib_onedir> - a compiled library in a single directory

=item C<cxx_lib_onedir> - a C++ compiled library in a single directory

=item C<lib_multidir> - a compiled library split into subsections

=back

=back



=item B<snippets>

=for brief check and optionally update snippets

check and optionally update snippets

Options:

=over

=item B<--copy>

Copy snippets to the local directory if they do not exist locally or
are out of date.

=item B<--diff>|B<-D>

If local snippets are out of date, print their differences.

=item B<--exclude> I<pattern>

Exclude files which match the pattern.  This uses a slightly extended
version of the standard shell file-matching pattern, where the C<*>
operator can match C</> characters. For example:

  --exclude 'templates/*'
  --exclude 'snippet_dist/*'

This option may be repeated.

=item B<--force>

Force snippets to be copied from the snippet repository, regardless
of their state of up-to-dateness.

=item B<--xsnip>

If local snippets are out of date, exit with a non-zero value.
The B<--copy> and B<--diff> options are still obeyed if specified.

=item B<--dir> I<directory list>

A colon separated path of directories to search for snippets.  If not
specified, it defaults to C</proj/axaf/simul/share/mstbuild/snippets>.  Use an extra colon
to insert the default directory in the list, e.g.

  /my/snippets/path::

resolves to

 /my/snippets/path:/proj/axaf/simul/share/mstbuild/snippets


=back

=item B<boot>

=for brief update snippets and perform autotool bootstrap

alias: bootstrap

run B<libtoolize>, B<aclocal>, B<autoheader>, B<automake>, and B<autoconf>.


Options:

=over

=item B<--aclocal_dir> I<path list>

This is a colon separated list of directories which contain local
B<autconf> macros.  Note that F</proj/axaf/simul/share/aclocal> is
automatically searched


=item B<--autoconf> I<alternative autoconf>

This specifies an alternative version of B<autoconf> to use. By
default it uses what is in the execution path.

=item B<--automake> I<alternative automake>

This specifies an alternative version of B<automake> to use. By
default it uses what is in the execution path.

=item B<--aclocal> I<alternative aclocal>

This specifies an alternative version of B<aclocal> to use. By
default it uses what is in the execution path.

=item B<--autoheader> I<alternative autoheader>

This specifies an alternative version of B<autoheader> to use. By
default it uses what is in the execution path.

=item B<--libtoolize> I<alternative libtoolize>

This specifies an alternative version of B<libtoolize> to use. By
default it uses what is in the execution path.

=item B<--autorun> I<list of commands>

This option limits the list of commands to run to those given.
The following commands are recognizes:

  autoconf automake aclocal autoheader libtoolize


=back

=item B<config>

=for brief run autoconf configure script

alias: configure

run B<configure>.

Options:

=over

=item B<--args> I<argument list>

Specify additional arguments to B<configure>. This option may be
repeated. The value of the following options are interpolated into the
arguments

  root
  buildtarget
  prefix
  exec_prefix
  destdir


=item B<--cache> | B<--nocache>

Turn caching by configure on or off.  It is on by default.

=back

=item B<make>

=for brief compile

run C<make>

=item B<check>

=for brief run tests

run C<make check>

=item B<chkstds>

=for brief check packaging consistency

B<chkstds> checks for packaging consistency.  The following
checks are made:

=over

=item required files in the package:

A license file, either C<COPYING> or C<LICENSE>, and
C<README>,
C<NEWS>,
and 
C<INSTALL>
files

=item versioned NEWS and ChangeLog files:

The version number in the C<configure.ac> file must appear in the
first few lines of C<NEWS> and C<ChangeLog>.

=back


Options:

Note C<chkstds> also takes template options. See L<Template Options>
or run

  mstbuild help options template


=over

=item B<--create>

Create missing files from templates.  A missing C<ChangeLog> file cannot
be created.  Be sure to edit the C<README> file.

=back

=item B<dist>

=for brief create distribution tarball

This first executes the B<chkstds> command, then 
runs C<make dist>

=item B<distcheck>

=for brief check distribution tarball

This first executes the B<chkstds> command, then
runs C<make distcheck>

=item B<export>

=for brief create and export distribution tarball

This command copies the distribution tarball to the export
directory.  If the B<--distcheck> option is specified, B<distcheck> is
first run.  If B<distcheck> is not run, the tarball must already exist.

Options:

=over

=item B<--dir|-d> F<directory>

The export directory.  This defaults to F</proj/axaf/simul/export>.

=item B<--distcheck>|B<--nodistcheck>

Run B<distcheck> first, which will create the tarball. This is the
default behaviour.

=item B<--ok>|B<--no-ok>

If set (the default) it's OK to generate the distribution tarball.
This option is mainly useful in the F<mstbuild.cfg> file to prevent
generation of a tarball for "proprietary" packages which should never
be exported.

=back

=item B<install>

=for brief install package

run C<make install>.

=item B<graft>

=for brief graft package

run

  graft -nit ${prefix} ${destdir}/${prefix}

if there are any conflicts, exit and allow the user to resolve them.
otherwise, run

  graft -it ${prefix} ${destdir}/${prefix}

=item B<clean>

=for brief run:  make clean

run C<make clean>

=item B<distclean>

=for brief run:  make distclean

run C<make distclean>

=item B<maintclean>

=for brief run:  make maintainer-clean

alias: maintainer-clean

run C<make maintainer-clean>

=item B<configplus>

=for brief execute commands from  configure to distcheck

execute commands: configure + make + check + distcheck

=item B<most>

=for brief execute commands from  bootstrap to distcheck

execute commands: bootstrap + configure + make + check + distcheck

=item B<all>

=for brief execute commands from bootstrap to graft to export

execute commands: bootstrap + configure + make + check + distcheck + install + graft + export

=item B<dumpenv>

=for brief output selected environmental variables

output selected environmental variables.

Options:

=over

=item B<--all>

Output all of the environmental variables, not just the select few.

=item B<--shell>|B<-s> I<shell>

set the shell syntax to be used for the B<dumpenv> command. Possible
values are:
C<bash>,
C<sh>,
C<ksh>,
C<csh>,
C<tcsh>.

It defaults to the contents of the C<SHELL> environmental variable.

=back

=item B<optvals>

=for brief output values of the global options

output values of the global options after applying any command line switches.

=back

=head1 Configuration file

B<mstbuild> reads a configuration file at startup, if available.  By
default it looks for F<mstbuild.cfg> in the current directory, but may
be directed elsewhere with the B<--configfile> option.  Configuration
files use an Apache style syntax (for indepth information, see
L<Config::General>).  There are two main constructs:

=over

=item Option value assignment:

  option_name = value

where I<option_name> is the long form of one of the command line
options.  Boolean options should be given a value of C<1> or C<0> or
C<true> or C<false>, e.g.

  arch = false

=item Block declaration:

There are unnamed blocks:

  <Directive>
    ...
  </Directive>

and named blocks

  <Directive name>
  </Directive>


Blocks may be nested.  Option value assignment may take place within blocks.

=back

Global configuration options value assignments are generally made
outside of any blocks (see the documentation on C<BuildTarget> blocks
for an exception).

The following blocks are recognized:

=over

=item B<BuildTarget>

These named blocks are used to specify options specific to a
particular platform.  The block names should be specific platform
designations (although they may contain glob wildcards
(e.g. C<x86_64-Linux_Debian*>) and are matched against the B<sysarch>
generated C<platform_generic>, C<platform_native>, C<os_type> and C<OS>
variables.

The enclosed values are applied after the options outside of the
B<BuildTarget> block.  C<buildtarget> options are applied after OS
options.  Global option assignments should appear at the outermost
level of this block. Options which may be specified only once will
override options specified outside the B<BuildTarget> block.  Options
which may be specified multiple times (such as B<--define> and
B<--makeflags>) are I<added>.

For example,

  <BuildTarget athlon_64>
    makeflags = -march athlon
  </BuildTarget>

would set the global B<--makeflags> option for B<athlon_64>
buildtargets.


=item B<define>

This block contains values for the B<--define> option.  The block does
not take a name.  For example, 

  --define a=b --define foo=goo

may be represented as

  <define>
     a = b
     foo = goo
  </define>

B<define> blocks may be placed inside B<BuildTarget> blocks.

=item B<Command>

These blocks are named with a B<mstbuild> command, and are used to
specify command specific options.

For example,

  <Command snippets>
    snippet_dir  = snippets_dist
    exclude      = templates/*
    exclude      = snippets_dist/*
  </Command>

would set options for the B<snippets> command.

B<Command> blocks may be placed inside B<BuildTarget> blocks.

=back


=head1 EXAMPLES

=over

=item To bootstrap, configure, make, make check, make install, and graft a package:

  mstbuild bootstrap configure make check install graft

=back

=head1 Magic

This script performs a whole lot of magic in order to ensure that
programs and libraries will appear in the F<$ROOT>
directory structure while actually living in separate, package and
version specific directories in F<$ROOT/pkgs>.

The packages and libraries are installed in F<$ROOT/pkgs>,
and then grafted onto F<$ROOT>.  We have to massage things
behind the scenes so that operationally none of the packages knows
about F<$ROOT/pkgs>, just F<$ROOT>.

This is a cursory summary of what happens here.  Read the B<autoconf>,
B<automake>, B<libtool>, and B<pkg-config> docs if you want a more in
depth understanding, i.e. this makes you say, 'eh?'.

The B<--destdir>, B<--exec_prefix>, and B<--prefix> options combine
to specify where the package will actually be installed.  This is
where the true magic is...

B<--destdir> specifies the C<DESTDIR> in the B<autoconf>/B<automake>
sense, a directory used to stage the install... not the final
location, but a step along the way.  B<--exec_prefix> and B<--prefix>
provide package specific subdirectories within B<--destdir>.  So
despite the fact that they are handed to C<install> in the
I<prefix> and I<exec_prefix> variables, they interact with C<DESTDIR>
in such a way that they are interpreted as subdirectories of
B<--destdir>.  Even if they are specified with a leading /, i.e. they
are absolute pathes, they are still treated as subdirectories of
B<--destdir>.

Confused?  Good.  Now we are in the same boat.

=head1 ENVIRONMENTAL VARIABLES

The following environmental variables are used, if available:

  MSTBUILD_ARGS MSTBUILD_ARGS0 CONFIG_SITE
  MAKE CC CXX F77 CFLAGS CXXFLAGS FFLAGS

The second batch will be ignored if the B<--nouenv> flag is specfied.

To dump the environment variables that may be set or modified by this
code, use the B<dumpenv> command.


=head2 PKG_CONFIG_PATH

B<PKG_CONFIG_PATH> is treated specially.  If the B<MST_Env> module is
available, an existing B<PKG_CONFIG_PATH> variable is ignored.  If the
B<MST_Env> module is not available, an existing variable is used.  The
B<--define> option may be used to override this behavior; it will
replace any previous or generated value of B<PKG_CONFIG_PATH>.

To add a path to an existing path, use the B<--pdefine> option.  This
option is processed after the B<--define> option.


=head2 Glossary

=over

=item CONFIG_SITE

B<autoconf> site wide configuration file

=item OS

the OS

=item MAKE

make executable to use.

=item CC

C compiler

=item CXX

C++ compiler

=item F77

Fortran 77 compiler

=item CFLAGS

C compiler flags

=item CXXFLAGS

C++ compiler flags

=item FFLAGS

Fortran 77 compiler flags

=back


=head1 EXAMPLES

=head2 Advanced Usage


=head3 Debugging a library using an application

Say that you need to compile against a library which has been compiled
with debugging turned on, and which uses B<pkg-config>.  Let's
say it's another B<mstbuild> enabled compile.

The scheme is to install the library in a temporary spot, then coax
the executable compilation to use it.  Let C<$ROOT> be the location of
the temporary directory.

=over

=item 1

Build and install the library with the following non-standard
B<mstbuild> flags:

  --root=$ROOT --graft=no -D CFLAGS=-g -D FFLAGS=-g

This creates a simple directory structure.  Because B<mstbuild> uses
B<--root> to find dependencies, if the library requires other
libraries, you'll have to explicitly specify the B<pkg-config> search
directories via the B<--pdefine> option:

  --pdefine PKG_CONFIG_PATH=-$OTHERROOT/lib/pkgconfig

=item 2

Build the program, adding the correct path to the B<pkg-config> metadata files:

  --graft=no \
  --pdefine PKG_CONFIG_PATH=-$ROOT/lib/pkgconfig

If the executable has library dependencies in $OTHERROOT, append the F<$OTHERROOT/lib/pkgconfig> to the B<--pdefine> option value:

  --graft=no \
  --pdefine \
    PKG_CONFIG_PATH=-$ROOT/lib/pkgconfig:$OTHERROOT/lib/pkgconfig


=back

=head3 Compile and install against a non-MST tree when B<MST_Env> is available.

Normally B<MST_Env> is used if it's available.  If, however, you're
working with dependencies which are too hot for the standard MST
install and which are installed in another directory tree, =mstbuild=
should ignore the MST environment.  This example assumes that the
other tree is complete, that all dependencies are found there and that
grafting is not used.

There are the options that should be set (B<$ROOT> is the alternate
directory tree)

  --graft=no
  --root $ROOT
  --nomstenv
  --define PKG_CONFIG_PATH=$ROOT/lib/pkgconfig


=head1 COPYRIGHT AND LICENSE

This software is Copyright The Smithsonian Astrophysical Observatory
and is released under the GNU General Public License.  You may find a
copy at: L<http://www.fsf.org/copyleft/gpl.html>

=head1 VERSION

This documents version 3.2.23 of B<mstbuild>.

=head1 AUTHORS

Mike Tibbetts

Diab Jerius E<lt>djerius@cfa.harvard.eduE<gt>
